/**
  ******************************************************************************
  * @ļ    stm32f10x_rcc.c
  * @    MCD Ӧó
  * @汾    V3.1.0
  * @    06/19/2009
  * @brief   ļ ṩ RCC ̼.
  * @    ANSON/sweet1985 Email:airanson110@126.com
  * @    03/07/2009
  ******************************************************************************
  * @
  *
  * ̼ṩͻΪƲƷдĲοĿʹͻԼʱ䡣
  * ͻʹñ̼ڿƷϲĽⷨ뵼幫˾еκֱӵĺͼ
  * ΣҲеκ𺦶⳥
  *
  * <h2><center>&; Ȩ 2009 ⷨ뵼幫˾</center></h2>
  * 汾ѧϰӢԭгӦӢԭΪ׼
  */ 

/*  ------------------------------------------------------------------*/
#include "stm32f10x_rcc.h"

/*  --------------------------------------------------------------*/
/* ö --------------------------------------------------------------*/

/* ------------ RCC registers bit address in the alias region ----------- */
#define RCC_OFFSET                (RCC_BASE - PERIPH_BASE)

/* --- CR Ĵ ---*/

/* Alias word address of HSION bit */
#define CR_OFFSET                 (RCC_OFFSET + 0x00)
#define HSION_BitNumber           0x00
#define CR_HSION_BB               (PERIPH_BB_BASE + (CR_OFFSET * 32) + (HSION_BitNumber * 4))

/* Alias word address of PLLON bit */
#define PLLON_BitNumber           0x18
#define CR_PLLON_BB               (PERIPH_BB_BASE + (CR_OFFSET * 32) + (PLLON_BitNumber * 4))

#ifdef STM32F10X_CL
 /* Alias word address of PLL2ON bit */
 #define PLL2ON_BitNumber          0x1A
 #define CR_PLL2ON_BB              (PERIPH_BB_BASE + (CR_OFFSET * 32) + (PLL2ON_BitNumber * 4))

 /* Alias word address of PLL3ON bit */
 #define PLL3ON_BitNumber          0x1C
 #define CR_PLL3ON_BB              (PERIPH_BB_BASE + (CR_OFFSET * 32) + (PLL3ON_BitNumber * 4))
#endif /* STM32F10X_CL */ 

/* Alias word address of CSSON bit */
#define CSSON_BitNumber           0x13
#define CR_CSSON_BB               (PERIPH_BB_BASE + (CR_OFFSET * 32) + (CSSON_BitNumber * 4))

/* --- CFGR Ĵ ---*/

/* Alias word address of USBPRE bit */
#define CFGR_OFFSET               (RCC_OFFSET + 0x04)

#ifndef STM32F10X_CL
 #define USBPRE_BitNumber          0x16
 #define CFGR_USBPRE_BB            (PERIPH_BB_BASE + (CFGR_OFFSET * 32) + (USBPRE_BitNumber * 4))
#else
 #define OTGFSPRE_BitNumber        0x16
 #define CFGR_OTGFSPRE_BB          (PERIPH_BB_BASE + (CFGR_OFFSET * 32) + (OTGFSPRE_BitNumber * 4))
#endif /* STM32F10X_CL */ 

/* --- BDCR Ĵ ---*/

/* Alias word address of RTCEN bit */
#define BDCR_OFFSET               (RCC_OFFSET + 0x20)
#define RTCEN_BitNumber           0x0F
#define BDCR_RTCEN_BB             (PERIPH_BB_BASE + (BDCR_OFFSET * 32) + (RTCEN_BitNumber * 4))

/* Alias word address of BDRST bit */
#define BDRST_BitNumber           0x10
#define BDCR_BDRST_BB             (PERIPH_BB_BASE + (BDCR_OFFSET * 32) + (BDRST_BitNumber * 4))

/* --- CSR Ĵ ---*/

/* Alias word address of LSION bit */
#define CSR_OFFSET                (RCC_OFFSET + 0x24)
#define LSION_BitNumber           0x00
#define CSR_LSION_BB              (PERIPH_BB_BASE + (CSR_OFFSET * 32) + (LSION_BitNumber * 4))

#ifdef STM32F10X_CL
/* --- CFGR2 Ĵ ---*/

 /* Alias word address of I2S2SRC bit */
 #define CFGR2_OFFSET              (RCC_OFFSET + 0x2C)
 #define I2S2SRC_BitNumber         0x11
 #define CFGR2_I2S2SRC_BB          (PERIPH_BB_BASE + (CFGR2_OFFSET * 32) + (I2S2SRC_BitNumber * 4))

 /* Alias word address of I2S3SRC bit */
 #define I2S3SRC_BitNumber         0x12
 #define CFGR2_I2S3SRC_BB          (PERIPH_BB_BASE + (CFGR2_OFFSET * 32) + (I2S3SRC_BitNumber * 4))
#endif /* STM32F10X_CL */

/* ---------------------- RCC Ĵ bit mask ------------------------ */

/* CR register bit mask */
#define CR_HSEBYP_Reset           ((uint32_t)0xFFFBFFFF)
#define CR_HSEBYP_Set             ((uint32_t)0x00040000)
#define CR_HSEON_Reset            ((uint32_t)0xFFFEFFFF)
#define CR_HSEON_Set              ((uint32_t)0x00010000)
#define CR_HSITRIM_Mask           ((uint32_t)0xFFFFFF07)

/* CFGR register bit mask */
#ifndef STM32F10X_CL
 #define CFGR_PLL_Mask            ((uint32_t)0xFFC0FFFF)
#else
 #define CFGR_PLL_Mask           ((uint32_t)0xFFC2FFFF)
#endif /* STM32F10X_CL */ 

#define CFGR_PLLMull_Mask         ((uint32_t)0x003C0000)
#define CFGR_PLLSRC_Mask          ((uint32_t)0x00010000)
#define CFGR_PLLXTPRE_Mask        ((uint32_t)0x00020000)
#define CFGR_SWS_Mask             ((uint32_t)0x0000000C)
#define CFGR_SW_Mask              ((uint32_t)0xFFFFFFFC)
#define CFGR_HPRE_Reset_Mask      ((uint32_t)0xFFFFFF0F)
#define CFGR_HPRE_Set_Mask        ((uint32_t)0x000000F0)
#define CFGR_PPRE1_Reset_Mask     ((uint32_t)0xFFFFF8FF)
#define CFGR_PPRE1_Set_Mask       ((uint32_t)0x00000700)
#define CFGR_PPRE2_Reset_Mask     ((uint32_t)0xFFFFC7FF)
#define CFGR_PPRE2_Set_Mask       ((uint32_t)0x00003800)
#define CFGR_ADCPRE_Reset_Mask    ((uint32_t)0xFFFF3FFF)
#define CFGR_ADCPRE_Set_Mask      ((uint32_t)0x0000C000)

/* CSR register bit mask */
#define CSR_RMVF_Set              ((uint32_t)0x01000000)

#ifdef STM32F10X_CL
/* CFGR2 register bit mask */
 #define CFGR2_PREDIV1SRC         ((uint32_t)0x00010000)
 #define CFGR2_PREDIV1            ((uint32_t)0x0000000F)
 #define CFGR2_PREDIV2            ((uint32_t)0x000000F0)
 #define CFGR2_PLL2MUL            ((uint32_t)0x00000F00)
 #define CFGR2_PLL3MUL            ((uint32_t)0x0000F000)
#endif /* STM32F10X_CL */ 

/* RCC Flag Mask */
#define FLAG_Mask                 ((uint8_t)0x1F)

#ifndef HSI_Value
/* Typical Value of the HSI in Hz */
 #define HSI_Value                 ((uint32_t)8000000)
#endif /* HSI_Value */

/* CIR register byte 2 (Bits[15:8]) base address */
#define CIR_BYTE2_ADDRESS         ((uint32_t)0x40021009)

/* CIR register byte 3 (Bits[23:16]) base address */
#define CIR_BYTE3_ADDRESS         ((uint32_t)0x4002100A)

/* CFGR register byte 4 (Bits[31:24]) base address */
#define CFGR_BYTE4_ADDRESS        ((uint32_t)0x40021007)

/* BDCR register base address */
#define BDCR_ADDRESS              (PERIPH_BASE + BDCR_OFFSET)

#ifndef HSEStartUp_TimeOut
/* Time out for HSE start up */
 #define HSEStartUp_TimeOut        ((uint16_t)0x0500)
#endif /* HSEStartUp_TimeOut */

/* ú ------------------------------------------------------------*/
/* ñ ----------------------------------------------------------*/

static __I uint8_t APBAHBPrescTable[16] = {0, 0, 0, 0, 1, 2, 3, 4, 1, 2, 3, 4, 6, 7, 8, 9};
static __I uint8_t ADCPrescTable[4] = {2, 4, 6, 8};

/* úԭ ------------------------------------------------------*/
/* ú ----------------------------------------------------------*/


/**
  * @  RCCĴΪȱʡֵ.
  * @  û
  * @  û
  */
void RCC_DeInit(void)
{
  /* λHSION (ڲ<8MHz>ʱʹ) */
  RCC->CR |= (uint32_t)0x00000001;

  /* λ SW, HPRE, PPRE1, PPRE2, ADCPRE , MCO λ */
  /* (HISѡϵͳʱӣHSIϵͳʱӣSYSCLKδƵHCLKδƵ
     ADCԤƵ<PLCK2>2ƵHSIʱ/2ѡPLLʱ)΢ʱ */
#ifndef STM32F10X_CL
  RCC->CFGR &= (uint32_t)0xF8FF0000;   /* ?ڱ1 */
#else
  RCC->CFGR &= (uint32_t)0xF0FF0000;   /* ?ڱ0 */
#endif /* STM32F10X_CL */   
  
  /* λ HSEON, CSSON, PLLON λ (PLLãʱӼãHSE) */
  RCC->CR &= (uint32_t)0xFEF6FFFF;

  /* λ HSEBYP λ (ⲿ1~25MHzδ·) */
  RCC->CR &= (uint32_t)0xFFFBFFFF;

  /* λ PLLSRC, PLLXTPRE, PLLMUL , USBPRE/OTGFSPRE λ */
  /*(USBԤƵΪPLLʱ1.5ƵPLLƵX2HSEδƵΪPLL룬HSIʱ/2PLL) */
  RCC->CFGR &= (uint32_t)0xFF80FFFF;

#ifndef STM32F10X_CL
  /* ʹʱжʧ */
  RCC->CIR = 0x009F0000;
#else
  /* λ PLL2ON , PLL3ON λ () */
  RCC->CR &= (uint32_t)0xEBFFFFFF;

  /* ʹʱжʧ  */
  RCC->CIR = 0x00FF0000;

  /* λ CFGR2 Ĵ */
  RCC->CFGR2 = 0x00000000;
#endif /* STM32F10X_CL */
}

/**
  * @  ⲿ (HSE).
  * @ע   HSE ֱʹûͨ PLL Ϊϵͳʱӣֹͣ.
  * @  RCC_HSE: ָ HSE µ״̬.
  *                 ѡһ:
  *             RCC_HSE_OFF: HSE  OFF
  *             RCC_HSE_ON : HSE  ON
  *             RCC_HSE_Bypass: HSE ·ʹⲿʱ
  * @  û
  */
void RCC_HSEConfig(uint32_t RCC_HSE)
{
  /*  */
  assert_param(IS_RCC_HSE(RCC_HSE));
  /* λ HSEON , HSEBYP λ  HSE ǰ ------------------*/
  /* Reset HSEON bit ( HSE )*/
  RCC->CR &= CR_HSEON_Reset;
  /* Reset HSEBYP bit (ⲿ1~25MHzδ·)*/
  RCC->CR &= CR_HSEBYP_Reset;
  /*  HSE (RCC_HSE_OFF ѾcodeĴ븲) */
  switch(RCC_HSE)
  {
    case RCC_HSE_ON:
      /* Set HSEON bit ( HSE )*/
      RCC->CR |= CR_HSEON_Set;
      break;
      
    case RCC_HSE_Bypass:
      /* Set HSEBYP and HSEON bits ( HSE ⲿ1~25MHzⲿʱ·)*/
      RCC->CR |= CR_HSEBYP_Set | CR_HSEON_Set;
      break;
      
    default:
      break;
  }
}

/**
  * @  ȴ HSE .
  * @  û
  * @  һErrorStatusöֵ:
  *        SUCCESS: HSE Ѿȶҿʹ
  *        ERROR  : HSE δ׼
  */
ErrorStatus RCC_WaitForHSEStartUp(void)
{
  __IO uint32_t StartUpCounter = 0;
  ErrorStatus status = ERROR;
  FlagStatus HSEStatus = RESET;
  
  /* ȴ HSE ׼úͳʱʱ䵽 */
  do
  {
    HSEStatus = RCC_GetFlagStatus(RCC_FLAG_HSERDY);
    StartUpCounter++;  
  } while((StartUpCounter != HSEStartUp_TimeOut) && (HSEStatus == RESET));
  
  if (RCC_GetFlagStatus(RCC_FLAG_HSERDY) != RESET)
  {
    status = SUCCESS;
  }
  else
  {
    status = ERROR;
  }  
  return (status);
}

/**
  * @  ڲ (HSI) ֵ.
  * @  HSICalibrationValue: Уֵһ00x1Fֵ֮.
  * @  û
  */
void RCC_AdjustHSICalibrationValue(uint8_t HSICalibrationValue)
{
  uint32_t tmpreg = 0;
  /*  */
  assert_param(IS_RCC_CALIBRATION_VALUE(HSICalibrationValue));
  tmpreg = RCC->CR;
  /*  HSITRIM[4:0] λ (Ĭֵ16ʹ HIS ϸ8MHz) */
  tmpreg &= CR_HSITRIM_Mask;
  /*  HSITRIM[4:0] λ HSICalibrationValue ֵ */
  tmpreg |= (uint32_t)HSICalibrationValue << 3;
  /* 洢µֵ */
  RCC->CR = tmpreg;
}

/**
  * @  ʹܻʧڲ (HSI).
  * @ע   HSE ֱʹûͨ PLL Ϊϵͳʱӣֹͣ.
  * @  NewState: HSI ״̬. 
  *                 ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_HSICmd(FunctionalState NewState)
{
  /*  */
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  *(__IO uint32_t *) CR_HSION_BB = (uint32_t)NewState;
}

/**
  * @  PLLʱԴƵϵ.
  * @ע  Ӧ PLL ֹʱ.
  * @  RCC_PLLSource: ָ PLL ʱԴ.
  *                       STM32_ϵ΢, ȡеֵ:
  *                                      RCC_PLLSource_HSI_Div2: PLL ʱ HSI ʱӶƵ
  *                                      RCC_PLLSource_PREDIV1: PREDIV1 clock selected as PLL clock entry
  *                       _STM32_΢, ȡеֵ:
  *                                      RCC_PLLSource_HSI_Div2: PLL ʱ HSI ʱӶƵ
  *                                      RCC_PLLSource_HSE_Div1: PLL ʱ HSE ʱ
  *                                      RCC_PLLSource_HSE_Div2: PLL ʱ HSE ʱӶƵ
  * @  RCC_PLLMul: ָ PLL Ƶϵ.
  *                    STM32_Connectivity_line_devices,  RCC_PLLMul_x xȡ{[4,9], 6_5}
  *                    other_STM32_devices,  RCC_PLLMul_x xȡ:[2,16]
  * @  û
  */
void RCC_PLLConfig(uint32_t RCC_PLLSource, uint32_t RCC_PLLMul)
{
  uint32_t tmpreg = 0;

  /*  */
  assert_param(IS_RCC_PLL_SOURCE(RCC_PLLSource));
  assert_param(IS_RCC_PLL_MUL(RCC_PLLMul));

  tmpreg = RCC->CFGR;
  /*  PLLSRC, PLLXTPRE , PLLMUL[3:0] λ */
  tmpreg &= CFGR_PLL_Mask;
  /*  PLL λ */
  tmpreg |= RCC_PLLSource | RCC_PLLMul;
  /* 洢ֵ */
  RCC->CFGR = tmpreg;
}

/**
  * @  ʹܻʧ PLL.
  * @ע   PLL Ϊϵͳʱӣֹͣ.
  * @  NewState: PLL ״̬. ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_PLLCmd(FunctionalState NewState)
{
  /*  */
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  *(__IO uint32_t *) CR_PLLON_BB = (uint32_t)NewState;
}

#ifdef STM32F10X_CL
/**
  * @   PREDIV1 Ƶ.
  * @ע 
  *   - Ӧ PLL ֹʱ.
  *   - Ӧ STM32 ϵ΢.
  * @  RCC_PREDIV1_Source: ָ PREDIV1 ʱԴ.
  *                            ȡֵеһ:
  *                            RCC_PREDIV1_Source_HSE: HSE Ϊ PREDIV1 ʱ
  *                            RCC_PREDIV1_Source_PLL2: PLL2 Ϊ PREDIV1 ʱ
  * @  RCC_PREDIV1_Div: ѡ PREDIV1 Ƶϵ.
  *                          RCC_PREDIV1_Divx  xȡ:[1,16]
  * @  û
  */
void RCC_PREDIV1Config(uint32_t RCC_PREDIV1_Source, uint32_t RCC_PREDIV1_Div)
{
  uint32_t tmpreg = 0;
  
  /*  */
  assert_param(IS_RCC_PREDIV1_SOURCE(RCC_PREDIV1_Source));
  assert_param(IS_RCC_PREDIV1(RCC_PREDIV1_Div));

  tmpreg = RCC->CFGR2;
  /*  PREDIV1[3:0]  PREDIV1SRC λ */
  tmpreg &= ~(CFGR2_PREDIV1 | CFGR2_PREDIV1SRC);
  /* Set the PREDIV1 clock source and division factor */
  tmpreg |= RCC_PREDIV1_Source | RCC_PREDIV1_Div ;
  /* 洢ֵ */
  RCC->CFGR2 = tmpreg;
}


/**
  * @   PREDIV2 Ƶ.
  * @ע 
  *   - Ӧ PLL2,PLL3 ֹʱ.
  *   - Ӧ STM32 ϵ΢.
  * @  RCC_PREDIV2_Div: ѡ PREDIV2 Ƶϵ.
  *                          RCC_PREDIV2_Divx  xȡ:[1,16]
  * @  û
  */
void RCC_PREDIV2Config(uint32_t RCC_PREDIV2_Div)
{
  uint32_t tmpreg = 0;

  /* Check the parameters */
  assert_param(IS_RCC_PREDIV2(RCC_PREDIV2_Div));

  tmpreg = RCC->CFGR2;
  /* Clear PREDIV2[3:0] bits */
  tmpreg &= ~CFGR2_PREDIV2;
  /* Set the PREDIV2 division factor */
  tmpreg |= RCC_PREDIV2_Div;
  /* Store the new value */
  RCC->CFGR2 = tmpreg;
}

/**
  * @   PLL2 Ƶϵ.
  * @ע
  *   - Ӧ PLL2 ֹʱ.
  *   - Ӧ STM32 ϵ΢.
  * @  RCC_PLL2Mul: ָ PLL2 Ƶϵ.
  *                      RCC_PLL2Mul_x  xȡ:{[8,14], 16, 20}
  * @  û
  */
void RCC_PLL2Config(uint32_t RCC_PLL2Mul)
{
  uint32_t tmpreg = 0;

  /*  */
  assert_param(IS_RCC_PLL2_MUL(RCC_PLL2Mul));

  tmpreg = RCC->CFGR2;
  /*  PLL2Mul[3:0] λ */
  tmpreg &= ~CFGR2_PLL2MUL;
  /*  PLL2 λ */
  tmpreg |= RCC_PLL2Mul;
  /* 洢 */
  RCC->CFGR2 = tmpreg;
}


/**
  * @  ʹܻʧ PLL2.
  * @ע 
  *   -  PLL2 ϵͳʱӣͲֹܱ
  *     (i.e. PLL ʱԴ˵ϵͳʱ).
  *   - Ӧ STM32 ϵ΢.
  * @  NewState: PLL2 µ״̬. ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_PLL2Cmd(FunctionalState NewState)
{
  /*  */
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  *(__IO uint32_t *) CR_PLL2ON_BB = (uint32_t)NewState;
}


/**
  * @   PLL3 Ƶϵ.
  * @ע
  *   - Ӧ PLL3 ֹʱ.
  *   - Ӧ STM32 ϵ΢.
  * @  RCC_PLL3Mul: ָ PLL3 Ƶϵ.
  *                      RCC_PLL3Mul_x  xȡ:{[8,14], 16, 20}
  * @  û
  */
void RCC_PLL3Config(uint32_t RCC_PLL3Mul)
{
  uint32_t tmpreg = 0;

  /*  */
  assert_param(IS_RCC_PLL3_MUL(RCC_PLL3Mul));

  tmpreg = RCC->CFGR2;
  /*  PLL3Mul[3:0] λ */
  tmpreg &= ~CFGR2_PLL3MUL;
  /*  PLL3 λ */
  tmpreg |= RCC_PLL3Mul;
  /* 洢ֵ */
  RCC->CFGR2 = tmpreg;
}


/**
  * @  ʹܻʧ PLL3.
  * @ע  Ӧ STM32 ϵ΢.
  * @  NewState: PLL3 µ״̬. ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_PLL3Cmd(FunctionalState NewState)
{
  /* Check the parameters */

  assert_param(IS_FUNCTIONAL_STATE(NewState));
  *(__IO uint32_t *) CR_PLL3ON_BB = (uint32_t)NewState;
}
#endif /* STM32F10X_CL */

/**
  * @  ϵͳʱ (SYSCLK).
  * @  RCC_SYSCLKSource: ָΪϵͳʱӵʱԴ.
  *                          ѡеһ:
  *                          RCC_SYSCLKSource_HSI:   ѡ HSI Ϊϵͳʱ
  *                          RCC_SYSCLKSource_HSE:   ѡ HSE Ϊϵͳʱ
  *                          RCC_SYSCLKSource_PLLCLK:ѡ PLL Ϊϵͳʱ
  * @  û
  */
void RCC_SYSCLKConfig(uint32_t RCC_SYSCLKSource)
{
  uint32_t tmpreg = 0;
  /*  */
  assert_param(IS_RCC_SYSCLK_SOURCE(RCC_SYSCLKSource));
  tmpreg = RCC->CFGR;
  /*  SW[1:0] λ  */
  tmpreg &= CFGR_SW_Mask;
  /*  SW[1:0] λΪ RCC_SYSCLKSource ֵ */
  tmpreg |= RCC_SYSCLKSource;
  /* 洢ֵ */
  RCC->CFGR = tmpreg;
}

/**
  * @  ϵͳʱӵʱԴ.
  * @  û
  * @  ΪϵͳʱӵʱԴ. 
  *        ֵ֮еһ:
  *        0x00: HSI ϵͳʱ
  *        0x04: HSE ϵͳʱ
  *        0x08: PLL ϵͳʱ
  */
uint8_t RCC_GetSYSCLKSource(void)
{
  return ((uint8_t)(RCC->CFGR & CFGR_SWS_Mask));
}

/**
  * @   AHB ʱ (HCLK).
  * @  RCC_SYSCLK:  AHB ʱӷƵ. ʱԴϵͳʱ
  *                    ȡֵеһ:
  *                    RCC_SYSCLK_Div1: AHB clock = SYSCLK
  *                    RCC_SYSCLK_Div2: AHB clock = SYSCLK/2
  *                    RCC_SYSCLK_Div4: AHB clock = SYSCLK/4
  *                    RCC_SYSCLK_Div8: AHB clock = SYSCLK/8
  *                    RCC_SYSCLK_Div16: AHB clock = SYSCLK/16
  *                    RCC_SYSCLK_Div64: AHB clock = SYSCLK/64
  *                    RCC_SYSCLK_Div128: AHB clock = SYSCLK/128
  *                    RCC_SYSCLK_Div256: AHB clock = SYSCLK/256
  *                    RCC_SYSCLK_Div512: AHB clock = SYSCLK/512
  * @  û
  */
void RCC_HCLKConfig(uint32_t RCC_SYSCLK)
{
  uint32_t tmpreg = 0;
  /*  */
  assert_param(IS_RCC_HCLK(RCC_SYSCLK));
  tmpreg = RCC->CFGR;
  /*  HPRE[3:0] λ */
  tmpreg &= CFGR_HPRE_Reset_Mask;
  /*  HPRE[3:0] λΪ RCC_SYSCLK ֵ */
  tmpreg |= RCC_SYSCLK;
  /* 洢ֵ */
  RCC->CFGR = tmpreg;
}

/**
  * @  õ APB ʱ (PCLK1).
  * @  RCC_HCLK:  APB1 ʱӷƵ. ʱԴ AHB ʱ (HCLK).
  *                  ȡֵеһ:
  *                  RCC_HCLK_Div1: APB1 clock = HCLK
  *                  RCC_HCLK_Div2: APB1 clock = HCLK/2
  *                  RCC_HCLK_Div4: APB1 clock = HCLK/4
  *                  RCC_HCLK_Div8: APB1 clock = HCLK/8
  *                  RCC_HCLK_Div16: APB1 clock = HCLK/16
  * @  û
  */
void RCC_PCLK1Config(uint32_t RCC_HCLK)
{
  uint32_t tmpreg = 0;
  /*  */
  assert_param(IS_RCC_PCLK(RCC_HCLK));
  tmpreg = RCC->CFGR;
  /*  PPRE1[2:0] λ */
  tmpreg &= CFGR_PPRE1_Reset_Mask;
  /*  PPRE1[2:0] λΪ RCC_HCLK ֵ */
  tmpreg |= RCC_HCLK;
  /* 洢ֵ */
  RCC->CFGR = tmpreg;
}

/**
  * @  ø APB ʱ (PCLK2).
  * @  RCC_HCLK:  APB2 ʱӷƵ. ʱԴ AHB ʱ (HCLK).
  *                  ȡֵеһ:
  *                  RCC_HCLK_Div1: APB1 clock = HCLK
  *                  RCC_HCLK_Div2: APB1 clock = HCLK/2
  *                  RCC_HCLK_Div4: APB1 clock = HCLK/4
  *                  RCC_HCLK_Div8: APB1 clock = HCLK/8
  *                  RCC_HCLK_Div16: APB1 clock = HCLK/16
  * @  û
  */
void RCC_PCLK2Config(uint32_t RCC_HCLK)
{
  uint32_t tmpreg = 0;
  /*  */
  assert_param(IS_RCC_PCLK(RCC_HCLK));
  tmpreg = RCC->CFGR;
  /*  PPRE2[2:0] λ */
  tmpreg &= CFGR_PPRE2_Reset_Mask;
  /*  PPRE2[2:0] λΪ RCC_HCLK ֵ */
  tmpreg |= RCC_HCLK << 3;
  /* 洢ֵ */
  RCC->CFGR = tmpreg;
}

/**
  * @  ʹܻʧָ RCC ж.
  * @  RCC_IT: ʹܻʧܵRCCжԴ.
  * 
  *                 STM32_ϵ΢, ȡֵеһ:      
  *                                         RCC_IT_LSIRDY: LSI ׼ж
  *                                         RCC_IT_LSERDY: LSE ׼ж
  *                                         RCC_IT_HSIRDY: HSI ׼ж
  *                                         RCC_IT_HSERDY: HSE ׼ж
  *                                         RCC_IT_PLLRDY: PLL ׼ж
  *                                         RCC_IT_PLL2RDY: PLL2 ׼ж
  *                                         RCC_IT_PLL3RDY: PLL3 ׼ж
  * 
  *                    _STM32_΢, ȡֵеһ:      
  *                                         RCC_IT_LSIRDY: LSI ׼ж
  *                                         RCC_IT_LSERDY: LSE ׼ж
  *                                         RCC_IT_HSIRDY: HSI ׼ж
  *                                         RCC_IT_HSERDY: HSE ׼ж
  *                                         RCC_IT_PLLRDY: PLL ׼ж
  *       
  * @  NewState: RCC жµ״̬.
  *                   ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_ITConfig(uint8_t RCC_IT, FunctionalState NewState)
{
  /*  */
  assert_param(IS_RCC_IT(RCC_IT));
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  if (NewState != DISABLE)
  {
    /* ִRCC_CIR[12:8]λֽڴȡʹѡжs */
    *(__IO uint8_t *) CIR_BYTE2_ADDRESS |= RCC_IT;
  }
  else
  {
    /* ִRCC_CIR[12:8]λֽڴȡֹѡж */
    *(__IO uint8_t *) CIR_BYTE2_ADDRESS &= (uint8_t)~RCC_IT;
  }
}

#ifndef STM32F10X_CL
/**
  * @   USB ʱ (USBCLK).
  * @  RCC_USBCLKSource: ָ USB ʱԴ. ʱԴ PLL .
  *                          ȡֵеһ:
  *                          RCC_USBCLKSource_PLLCLK_1Div5: PLL ʱӵ1.5Ƶ USB ʱԴ
  *                          RCC_USBCLKSource_PLLCLK_Div1:  PLL ʱ USB ʱԴ
  * @  û
  */
void RCC_USBCLKConfig(uint32_t RCC_USBCLKSource)
{
  /*  */
  assert_param(IS_RCC_USBCLK_SOURCE(RCC_USBCLKSource));

  *(__IO uint32_t *) CFGR_USBPRE_BB = RCC_USBCLKSource;
}
#else
/**
  * @   USB OTG FS ʱ (OTGFSCLK).
  *        Ӧ STM32 ϵ΢.
  * @  RCC_OTGFSCLKSource: ָ USB OTG FS ʱԴ.
  *                            ʱԴ PLL .
  *                            ȡֵеһ:
  *                            RCC_OTGFSCLKSource_PLLVCO_Div3: PLL VCO clock divided by 2 selected as USB OTG FS clock source
  *                            RCC_OTGFSCLKSource_PLLVCO_Div2: PLL VCO clock divided by 2 selected as USB OTG FS clock source
  * @  û
  */
void RCC_OTGFSCLKConfig(uint32_t RCC_OTGFSCLKSource)
{
  /*  */
  assert_param(IS_RCC_OTGFSCLK_SOURCE(RCC_OTGFSCLKSource));

  *(__IO uint32_t *) CFGR_OTGFSPRE_BB = RCC_OTGFSCLKSource;
}
#endif /* STM32F10X_CL */ 

/**
  * @   ADC ʱ (ADCCLK).
  * @  RCC_PCLK2:  ADC ʱӷƵ. ʱԴ APB2 ʱ (PCLK2).
  *                                      ȡֵеһ:
  *                                      RCC_PCLK2_Div2: ADC clock = PCLK2/2
  *                                      RCC_PCLK2_Div4: ADC clock = PCLK2/4
  *                                      RCC_PCLK2_Div6: ADC clock = PCLK2/6
  *                                      RCC_PCLK2_Div8: ADC clock = PCLK2/8
  * @  û
  */
void RCC_ADCCLKConfig(uint32_t RCC_PCLK2)
{
  uint32_t tmpreg = 0;
  /*  */
  assert_param(IS_RCC_ADCCLK(RCC_PCLK2));
  tmpreg = RCC->CFGR;
  /*  ADCPRE[1:0] λ */
  tmpreg &= CFGR_ADCPRE_Reset_Mask;
  /*  ADCPRE[1:0] λȡ RCC_PCLK2 ֵ */
  tmpreg |= RCC_PCLK2;
  /* 洢ֵ */
  RCC->CFGR = tmpreg;
}

#ifdef STM32F10X_CL
/**
  * @   I2S2 ʱԴ(I2S2CLK).
  * @ע
  *   - ʹI2S2 APBʱ֮ǰ.
  *   - Ӧ STM32 ϵ΢.
  * @  RCC_I2S2CLKSource: ָ I2S2 ʱԴ.
  *                           ȡֵеһ:
  *                           RCC_I2S2CLKSource_SYSCLK: ϵͳʱΪ I2S2 ʱԴ
  *                           RCC_I2S2CLKSource_PLL3_VCO: PLL3 VCO Ϊ I2S2 ʱԴ
  * @  û
  */
void RCC_I2S2CLKConfig(uint32_t RCC_I2S2CLKSource)
{
  /*  */
  assert_param(IS_RCC_I2S2CLK_SOURCE(RCC_I2S2CLKSource));

  *(__IO uint32_t *) CFGR2_I2S2SRC_BB = RCC_I2S2CLKSource;
}

/**
  * @   I2S3 ʱԴ(I2S2CLK).
  * @ע
  *   - ʹI2S3 APBʱ֮ǰ.
  *   - Ӧ STM32 ϵ΢.
  * @  RCC_I2S3CLKSource: ָ I2S3 ʱԴ.
  *                           ȡֵеһ:
  *                           RCC_I2S2CLKSource_SYSCLK: ϵͳʱΪ I2S3 ʱԴ
  *                           RCC_I2S2CLKSource_PLL3_VCO: PLL3 VCO Ϊ I2S3 ʱԴ
  * @  û
  */
void RCC_I2S3CLKConfig(uint32_t RCC_I2S3CLKSource)
{
  /* ָ */
  assert_param(IS_RCC_I2S3CLK_SOURCE(RCC_I2S3CLKSource));

  *(__IO uint32_t *) CFGR2_I2S3SRC_BB = RCC_I2S3CLKSource;
}
#endif /* STM32F10X_CL */

/**
  * @  ⲿ (LSE).
  * @  RCC_LSE: ָ LSE ״̬.
  *                 ȡֵеһ:
  *                 RCC_LSE_OFF: LSE  OFF
  *                 RCC_LSE_ON: LSE  ON
  *                 RCC_LSE_Bypass: LSE ⲿʱ·
  * @  û
  */
void RCC_LSEConfig(uint8_t RCC_LSE)
{
  /*  */
  assert_param(IS_RCC_LSE(RCC_LSE));
  /* Reset LSEON and LSEBYP bits before configuring the LSE ------------------*/
  /* λ LSEON λ */
  *(__IO uint8_t *) BDCR_ADDRESS = RCC_LSE_OFF;
  /* λ LSEBYP λ */
  *(__IO uint8_t *) BDCR_ADDRESS = RCC_LSE_OFF;
  /* Configure LSE (RCC_LSE_OFF Ѿ code 벿ֱ) */
  switch(RCC_LSE)
  {
    case RCC_LSE_ON:
      /*  LSEON λ */
      *(__IO uint8_t *) BDCR_ADDRESS = RCC_LSE_ON;
      break;
      
    case RCC_LSE_Bypass:
      /*  LSEBYP , LSEON λ */
      *(__IO uint8_t *) BDCR_ADDRESS = RCC_LSE_Bypass | RCC_LSE_ON;
      break;            
      
    default:
      break;      
  }
}

/**
  * @  ʹܻʧڲ (LSI).
  * @ע   IWDG УLSI Ͳܱ.
  * @  NewState: LSI µ״̬. ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_LSICmd(FunctionalState NewState)
{
  /*  */
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  *(__IO uint32_t *) CSR_LSION_BB = (uint32_t)NewState;
}

/**
  * @   RTC ʱ (RTCCLK).
  * @ע  ֻҪѡ RTC ʱ,RTCʱӾͲܸıֱλ.
  * @  RCC_RTCCLKSource: ָ RTC ʱԴ.
  *                          ȡֵеһ:
  *                          RCC_RTCCLKSource_LSE: LSE Ϊ RTC ʱ
  *                          RCC_RTCCLKSource_LSI: LSI Ϊ RTC ʱ
  *                          RCC_RTCCLKSource_HSE_Div128: HSE ʱӵ 128 ƵΪ RTC ʱ
  * @  û
  */
void RCC_RTCCLKConfig(uint32_t RCC_RTCCLKSource)
{
  /*  */
  assert_param(IS_RCC_RTCCLK_SOURCE(RCC_RTCCLKSource));
  /* ָ RTC ʱԴ */
  RCC->BDCR |= RCC_RTCCLKSource;
}

/**
  * @  ʹܻʧ RTC ʱ.
  * @ע  ʹ RCC_RTCCLKConfig ѡ RTC ʱӺ,ʹ.
  * @  NewState: RTC ʱӵ״̬. ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_RTCCLKCmd(FunctionalState NewState)
{
  /*  */
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  *(__IO uint32_t *) BDCR_RTCEN_BB = (uint32_t)NewState;
}

/**
  * @  ƬʱӵĲͬƵ.
  * @  RCC_Clocks: ָʱƵʵ RCC_ClocksTypeDef ṹָ.
  * @  û
  */
void RCC_GetClocksFreq(RCC_ClocksTypeDef* RCC_Clocks)
{
  uint32_t tmp = 0, pllmull = 0, pllsource = 0, presc = 0;

#ifdef  STM32F10X_CL
  uint32_t prediv1source = 0, prediv1factor = 0, prediv2factor = 0, pll2mull = 0;
#endif /* STM32F10X_CL */
    
  /* õϵͳʱԴ -------------------------------------------------------*/
  tmp = RCC->CFGR & CFGR_SWS_Mask;
  
  switch (tmp)
  {
    case 0x00:  /* HSI used as system clock */
      RCC_Clocks->SYSCLK_Frequency = HSI_Value;
      break;
    case 0x04:  /* HSE used as system clock */
      RCC_Clocks->SYSCLK_Frequency = HSE_Value;
      break;
    case 0x08:  /* PLL used as system clock */

      /* õ PLL ʱԴͱƵ ----------------------*/
      pllmull = RCC->CFGR & CFGR_PLLMull_Mask;
      pllsource = RCC->CFGR & CFGR_PLLSRC_Mask;
      
#ifndef STM32F10X_CL      
      pllmull = ( pllmull >> 18) + 2;
      
      if (pllsource == 0x00)
      {/* HSI ʱӵ2ƵΪ PLL ʱԴ */
        RCC_Clocks->SYSCLK_Frequency = (HSI_Value >> 1) * pllmull;
      }
      else
      {/* HSE Ϊ PLL ʱԴ */
        if ((RCC->CFGR & CFGR_PLLXTPRE_Mask) != (uint32_t)RESET)
        {/* HSE ʱӱ2Ƶ */
          RCC_Clocks->SYSCLK_Frequency = (HSE_Value >> 1) * pllmull;
        }
        else
        {
          RCC_Clocks->SYSCLK_Frequency = HSE_Value * pllmull;
        }
      }
#else
      pllmull = pllmull >> 18;
      
      if (pllmull != 0x0D)
      {
         pllmull += 2;
      }
      else
      { /* PLL Ƶ = PLL ʱ * 6.5 */
        pllmull = 13 / 2; 
      }
            
      if (pllsource == 0x00)
      {/* HSI ʱӱ2ƵΪ PLL ʱԴ */
        RCC_Clocks->SYSCLK_Frequency = (HSI_Value >> 1) * pllmull;
      }
      else
      {/* PREDIV1 Ϊ PLL ʱԴ */
        
        /* ָ PREDIV1 ʱԴͷƵϵ */
        prediv1source = RCC->CFGR2 & CFGR2_PREDIV1SRC;
        prediv1factor = (RCC->CFGR2 & CFGR2_PREDIV1) + 1;
        
        if (prediv1source == 0)
        { /* HSE ʱӱѡΪ PREDIV1 ʱԴ */
          RCC_Clocks->SYSCLK_Frequency = (HSE_Value / prediv1factor) * pllmull;          
        }
        else
        {/* PLL2 ʱΪ PREDIV1 ʱԴ */
          
          /* ָ PREDIV2 Ƶϵ  PLL2 Ƶϵ */
          prediv2factor = ((RCC->CFGR2 & CFGR2_PREDIV2) >> 4) + 1;
          pll2mull = ((RCC->CFGR2 & CFGR2_PLL2MUL) >> 8 ) + 2; 
          RCC_Clocks->SYSCLK_Frequency = (((HSE_Value / prediv2factor) * pll2mull) / prediv1factor) * pllmull;                         
        }
      }
#endif /* STM32F10X_CL */ 
      break;

    default:
      RCC_Clocks->SYSCLK_Frequency = HSI_Value;
      break;
  }

  /*  HCLK, PCLK1, PCLK2 , ADCCLK ʱƵ ----------------*/
  /* õ HCLK ԤƵ */
  tmp = RCC->CFGR & CFGR_HPRE_Set_Mask;
  tmp = tmp >> 4;
  presc = APBAHBPrescTable[tmp];
  /* HCLK ʱƵ */
  RCC_Clocks->HCLK_Frequency = RCC_Clocks->SYSCLK_Frequency >> presc;
  /* õ PCLK1 ԤƵ */
  tmp = RCC->CFGR & CFGR_PPRE1_Set_Mask;
  tmp = tmp >> 8;
  presc = APBAHBPrescTable[tmp];
  /* PCLK1 ʱƵ */
  RCC_Clocks->PCLK1_Frequency = RCC_Clocks->HCLK_Frequency >> presc;
  /* õ PCLK2 ԤƵ */
  tmp = RCC->CFGR & CFGR_PPRE2_Set_Mask;
  tmp = tmp >> 11;
  presc = APBAHBPrescTable[tmp];
  /* PCLK2 ʱƵ */
  RCC_Clocks->PCLK2_Frequency = RCC_Clocks->HCLK_Frequency >> presc;
  /* õ ADCCLK ԤƵ */
  tmp = RCC->CFGR & CFGR_ADCPRE_Set_Mask;
  tmp = tmp >> 14;
  presc = ADCPrescTable[tmp];
  /* ADCCLK ʱƵ */
  RCC_Clocks->ADCCLK_Frequency = RCC_Clocks->PCLK2_Frequency / presc;
}

/**
  * @  ʹܻʧ AHB Χ豸ʱ.
  * @  RCC_AHBPeriph: ſʱӵ AHB Χ豸.
  *   
  *        STM32_ϵ΢, ȡֵеһ:      
  *                                RCC_AHBPeriph_DMA1
  *                                RCC_AHBPeriph_DMA2
  *                                RCC_AHBPeriph_SRAM
  *                                RCC_AHBPeriph_FLITF
  *                                RCC_AHBPeriph_CRC
  *                                RCC_AHBPeriph_OTG_FS    
  *                                RCC_AHBPeriph_ETH_MAC   
  *                                RCC_AHBPeriph_ETH_MAC_Tx
  *                                RCC_AHBPeriph_ETH_MAC_Rx
  * 
  *           _STM32_΢, ȡֵеһ:         
  *                                RCC_AHBPeriph_DMA1
  *                                RCC_AHBPeriph_DMA2
  *                                RCC_AHBPeriph_SRAM
  *                                RCC_AHBPeriph_FLITF
  *                                RCC_AHBPeriph_CRC
  *                                RCC_AHBPeriph_FSMC
  *                                RCC_AHBPeriph_SDIO
  *   
  * @ע  SRAM  FLITF ʱֻ˯ߵģʽ¿Ч.
  * @  NewState: ָΧ豸ʱµ״̬.
  *                  ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_AHBPeriphClockCmd(uint32_t RCC_AHBPeriph, FunctionalState NewState)
{
  /*  */
  assert_param(IS_RCC_AHB_PERIPH(RCC_AHBPeriph));
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    RCC->AHBENR |= RCC_AHBPeriph;
  }
  else
  {
    RCC->AHBENR &= ~RCC_AHBPeriph;
  }
}

/**
  * @  ʹܻʧܸ APB (APB2) Χ豸ʱ.
  * @  RCC_APB2Periph: ſʱӵ AHB2 Χ豸.
  *                        ֵ:
  *          RCC_APB2Periph_AFIO, RCC_APB2Periph_GPIOA, RCC_APB2Periph_GPIOB,
  *          RCC_APB2Periph_GPIOC, RCC_APB2Periph_GPIOD, RCC_APB2Periph_GPIOE,
  *          RCC_APB2Periph_GPIOF, RCC_APB2Periph_GPIOG, RCC_APB2Periph_ADC1,
  *          RCC_APB2Periph_ADC2, RCC_APB2Periph_TIM1, RCC_APB2Periph_SPI1,
  *          RCC_APB2Periph_TIM8, RCC_APB2Periph_USART1, RCC_APB2Periph_ADC3
  * @  NewState: ָΧ豸ʱӵ״̬.
  *                  ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_APB2PeriphClockCmd(uint32_t RCC_APB2Periph, FunctionalState NewState)
{
  /*  */
  assert_param(IS_RCC_APB2_PERIPH(RCC_APB2Periph));
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  if (NewState != DISABLE)
  {
    RCC->APB2ENR |= RCC_APB2Periph;
  }
  else
  {
    RCC->APB2ENR &= ~RCC_APB2Periph;
  }
}

/**
  * @  ʹܻʧܵ APB (APB1) Χ豸ʱ.
  * @  RCC_APB1Periph: ſʱӵ APB1 Χ豸.
  *                        ֵ:
  *          RCC_APB1Periph_TIM2, RCC_APB1Periph_TIM3, RCC_APB1Periph_TIM4,
  *          RCC_APB1Periph_TIM5, RCC_APB1Periph_TIM6, RCC_APB1Periph_TIM7,
  *          RCC_APB1Periph_WWDG, RCC_APB1Periph_SPI2, RCC_APB1Periph_SPI3,
  *          RCC_APB1Periph_USART2, RCC_APB1Periph_USART3, RCC_APB1Periph_USART4, 
  *          RCC_APB1Periph_USART5, RCC_APB1Periph_I2C1, RCC_APB1Periph_I2C2,
  *          RCC_APB1Periph_USB, RCC_APB1Periph_CAN1, RCC_APB1Periph_BKP,
  *          RCC_APB1Periph_PWR, RCC_APB1Periph_DAC
  * @  NewState: ָΧ豸ʱӵ״̬.
  *                  ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_APB1PeriphClockCmd(uint32_t RCC_APB1Periph, FunctionalState NewState)
{
  /*  */
  assert_param(IS_RCC_APB1_PERIPH(RCC_APB1Periph));
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  if (NewState != DISABLE)
  {
    RCC->APB1ENR |= RCC_APB1Periph;
  }
  else
  {
    RCC->APB1ENR &= ~RCC_APB1Periph;
  }
}

#ifdef STM32F10X_CL
/**
  * @  ƻͷ AHB Χ豸λ.
  * @ע  Ӧ STM32 ϵ΢.
  * @  RCC_AHBPeriph: ָλΧ豸.
  *                       ֵ:
  *                       RCC_AHBPeriph_OTG_FS 
  *                       RCC_AHBPeriph_ETH_MAC
  * @  NewState: ָΧ豸λ״̬.
  *                  ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_AHBPeriphResetCmd(uint32_t RCC_AHBPeriph, FunctionalState NewState)
{
  /*  */
  assert_param(IS_RCC_AHB_PERIPH_RESET(RCC_AHBPeriph));
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    RCC->AHBRSTR |= RCC_AHBPeriph;
  }
  else
  {
    RCC->AHBRSTR &= ~RCC_AHBPeriph;
  }
}
#endif /* STM32F10X_CL */ 

/**
  * @  ƻͷŸ APB(APB2) Χ豸λ.
  * @  RCC_APB2Periph: ָλ APB2 Χ豸.
  *                        ֵ:
  *          RCC_APB2Periph_AFIO, RCC_APB2Periph_GPIOA, RCC_APB2Periph_GPIOB,
  *          RCC_APB2Periph_GPIOC, RCC_APB2Periph_GPIOD, RCC_APB2Periph_GPIOE,
  *          RCC_APB2Periph_GPIOF, RCC_APB2Periph_GPIOG, RCC_APB2Periph_ADC1,
  *          RCC_APB2Periph_ADC2, RCC_APB2Periph_TIM1, RCC_APB2Periph_SPI1,
  *          RCC_APB2Periph_TIM8, RCC_APB2Periph_USART1, RCC_APB2Periph_ADC3
  * @  NewState: ָΧ豸λ״̬.
  *                  ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_APB2PeriphResetCmd(uint32_t RCC_APB2Periph, FunctionalState NewState)
{
  /*  */
  assert_param(IS_RCC_APB2_PERIPH(RCC_APB2Periph));
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  if (NewState != DISABLE)
  {
    RCC->APB2RSTR |= RCC_APB2Periph;
  }
  else
  {
    RCC->APB2RSTR &= ~RCC_APB2Periph;
  }
}

/**
  * @  ƻͷŵ APB(APB1) Χ豸λ.
  * @  RCC_APB1Periph: ָλ APB1 Χ豸.
  *                        ֵ:
  *          RCC_APB1Periph_TIM2, RCC_APB1Periph_TIM3, RCC_APB1Periph_TIM4,
  *          RCC_APB1Periph_TIM5, RCC_APB1Periph_TIM6, RCC_APB1Periph_TIM7,
  *          RCC_APB1Periph_WWDG, RCC_APB1Periph_SPI2, RCC_APB1Periph_SPI3,
  *          RCC_APB1Periph_USART2, RCC_APB1Periph_USART3, RCC_APB1Periph_USART4, 
  *          RCC_APB1Periph_USART5, RCC_APB1Periph_I2C1, RCC_APB1Periph_I2C2,
  *          RCC_APB1Periph_USB, RCC_APB1Periph_CAN1, RCC_APB1Periph_BKP,
  *          RCC_APB1Periph_PWR, RCC_APB1Periph_DAC
  * @  NewState: ָΧ豸λ״̬.
  *                  ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_APB1PeriphResetCmd(uint32_t RCC_APB1Periph, FunctionalState NewState)
{
  /*  */
  assert_param(IS_RCC_APB1_PERIPH(RCC_APB1Periph));
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  if (NewState != DISABLE)
  {
    RCC->APB1RSTR |= RCC_APB1Periph;
  }
  else
  {
    RCC->APB1RSTR &= ~RCC_APB1Periph;
  }
}

/**
  * @  ƻͷűλ.
  * @  NewState: λ״̬.
  *                  ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_BackupResetCmd(FunctionalState NewState)
{
  /*  */
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  *(__IO uint32_t *) BDCR_BDRST_BB = (uint32_t)NewState;
}

/**
  * @  ʹܻʧʱӰȫϵͳ.
  * @  NewState: ʱӰȫϵͳ״̬.
  *                  ȡ: ENABLE  DISABLE.
  * @  û
  */
void RCC_ClockSecuritySystemCmd(FunctionalState NewState)
{
  /*  */
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  *(__IO uint32_t *) CR_CSSON_BB = (uint32_t)NewState;
}

/**
  * @  ѡ MCO ϵʱԴ.
  * @  RCC_MCO: ָҪʱԴ.
  *   
  *        STM32_ϵ΢, ȡֵеһ:      
  *                                RCC_MCO_NoClock:      ûѡʱ
  *                                RCC_MCO_SYSCLK:       ѡϵͳʱ
  *                                RCC_MCO_HSI:          ѡHISʱ
  *                                RCC_MCO_HSE:          ѡHSEʱ
  *                                RCC_MCO_PLLCLK_Div2:  ѡPLLʱƵ
  *                                RCC_MCO_PLL2CLK:      ѡPLL2ʱ                    
  *                                RCC_MCO_PLL3CLK_Div2: ѡPLL3ʱƵ   
  *                                RCC_MCO_XT1:          ѡⲿ 3-25 MHz ʱ  
  *                                RCC_MCO_PLL3CLK:      ѡPLL3ʱ 
  * 
  *           _STM32_΢, ȡֵеһ:        
  *                                RCC_MCO_NoClock:      ûѡʱ
  *                                RCC_MCO_SYSCLK:       ѡϵͳʱ
  *                                RCC_MCO_HSI:          ѡHISʱ
  *                                RCC_MCO_HSE:          ѡHSEʱ
  *                                RCC_MCO_PLLCLK_Div2:  ѡPLLʱƵ
  *   
  * @  û
  */
void RCC_MCOConfig(uint8_t RCC_MCO)
{
  /*  */
  assert_param(IS_RCC_MCO(RCC_MCO));

  /* ִMCO[2:0]λֽڴȡѡMCOԴ */
  *(__IO uint8_t *) CFGR_BYTE4_ADDRESS = RCC_MCO;
}

/**
  * @  ָ RCC ־.
  * @  RCC_FLAG: ָҪı־.
  *   
  *        STM32_ϵ΢, ȡֵеһ: 
  *                                RCC_FLAG_HSIRDY:  HSI ʱӾ
  *                                RCC_FLAG_HSERDY:  HSE ʱӾ
  *                                RCC_FLAG_PLLRDY:  PLL ʱӾ
  *                                RCC_FLAG_PLL2RDY: PLL2 ʱӾ    
  *                                RCC_FLAG_PLL3RDY: PLL3 ʱӾ                          
  *                                RCC_FLAG_LSERDY:  LSE ʱӾ
  *                                RCC_FLAG_LSIRDY:  LSI ʱӾ
  *                                RCC_FLAG_PINRST:  Ÿλ
  *                                RCC_FLAG_PORRST:  POR/PDRλ
  *                                RCC_FLAG_SFTRST:  λ
  *                                RCC_FLAG_IWDGRST: Źλ
  *                                RCC_FLAG_WWDGRST: ڿŹλ
  *                                RCC_FLAG_LPWRRST: ͵λ
  * 
  *           _STM32_΢, ȡֵеһ:            
  *                                RCC_FLAG_HSIRDY:  HSI ʱӾ
  *                                RCC_FLAG_HSERDY:  HSE ʱӾ
  *                                RCC_FLAG_PLLRDY:  PLL ʱӾ
  *                                RCC_FLAG_LSERDY:  LSE ʱӾ
  *                                RCC_FLAG_LSIRDY:  LSI ʱӾ
  *                                RCC_FLAG_PINRST:  Ÿλ
  *                                RCC_FLAG_PORRST:  POR/PDRλ
  *                                RCC_FLAG_SFTRST:  λ
  *                                RCC_FLAG_IWDGRST: Źλ
  *                                RCC_FLAG_WWDGRST: ڿŹλ
  *                                RCC_FLAG_LPWRRST: ͵λ
  *   
  * @  RCC_FLAG ״̬(SETRESET).
  */
FlagStatus RCC_GetFlagStatus(uint8_t RCC_FLAG)
{
  uint32_t tmp = 0;
  uint32_t statusreg = 0;
  FlagStatus bitstatus = RESET;
  /*  */
  assert_param(IS_RCC_FLAG(RCC_FLAG));

  /* ȡ RCC Ĵ */
  tmp = RCC_FLAG >> 5;
  if (tmp == 1)               /* CR Ĵеļ־ */
  {
    statusreg = RCC->CR;
  }
  else if (tmp == 2)          /* BDCR Ĵеļ־ */
  {
    statusreg = RCC->BDCR;
  }
  else                       /* CSR Ĵеļ־ */
  {
    statusreg = RCC->CSR;
  }

  /* ȡñ־λ */
  tmp = RCC_FLAG & FLAG_Mask;
  if ((statusreg & ((uint32_t)1 << tmp)) != (uint32_t)RESET)
  {
    bitstatus = SET;
  }
  else
  {
    bitstatus = RESET;
  }

  /* ر־״̬ */
  return bitstatus;
}

/**
  * @   RCC λ־.
  * @ע   Щλ־: RCC_FLAG_PINRST, RCC_FLAG_PORRST, RCC_FLAG_SFTRST,
  *                         RCC_FLAG_IWDGRST, RCC_FLAG_WWDGRST, RCC_FLAG_LPWRRST
  * @  û
  * @  û
  */
void RCC_ClearFlag(void)
{
  /* ͨλ RMVF λλ־ */
  RCC->CSR |= CSR_RMVF_Set;
}

/**
  * @  ָ RCC жǷ.
  * @  RCC_IT: ָ RCC жԴ.
  *   
  *        STM32_ϵ΢, ȡֵеһ: 
  *                                RCC_IT_LSIRDY:  LSI ж
  *                                RCC_IT_LSERDY:  LSE ж
  *                                RCC_IT_HSIRDY:  HSI ж
  *                                RCC_IT_HSERDY:  HSE ж
  *                                RCC_IT_PLLRDY:  PLL ж
  *                                RCC_IT_PLL2RDY: PLL2 ж 
  *                                RCC_IT_PLL3RDY: PLL3 ж                    
  *                                RCC_IT_CSS:     ʱӰȫϵͳж
  * 
  *           _STM32_΢, ȡֵеһ:      
  *                                RCC_IT_LSIRDY: LSI ж
  *                                RCC_IT_LSERDY: LSE ж
  *                                RCC_IT_HSIRDY: HSI ж
  *                                RCC_IT_HSERDY: HSE ж
  *                                RCC_IT_PLLRDY: PLL ж
  *                                RCC_IT_CSS:    ʱӰȫϵͳж
  *   
  * @  RCC_IT ״̬(SETorRESET).
  */
ITStatus RCC_GetITStatus(uint8_t RCC_IT)
{
  ITStatus bitstatus = RESET;
  /*  */
  assert_param(IS_RCC_GET_IT(RCC_IT));

  /* ָ RCC ж״̬ */
  if ((RCC->CIR & RCC_IT) != (uint32_t)RESET)
  {
    bitstatus = SET;
  }
  else
  {
    bitstatus = RESET;
  }

  /*  RCC_IT ״̬ */
  return  bitstatus;
}

/**
  * @   RCC жϹλ.
  * @  RCC_IT: ָҪжϹλ.
  *   
  *        STM32_ϵ΢, ȡֵеһ: 
  *                                RCC_IT_LSIRDY:  LSI ж
  *                                RCC_IT_LSERDY:  LSE ж
  *                                RCC_IT_HSIRDY:  HSI ж
  *                                RCC_IT_HSERDY:  HSE ж
  *                                RCC_IT_PLLRDY:  PLL ж
  *                                RCC_IT_PLL2RDY: PLL2 ж
  *                                RCC_IT_PLL3RDY: PLL3 ж                      
  *                                RCC_IT_CSS:     ʱӰȫϵͳж
  * 
  *           _STM32_΢, ȡֵеһ:           
  *                                RCC_IT_LSIRDY:  LSI ж
  *                                RCC_IT_LSERDY:  LSE ж
  *                                RCC_IT_HSIRDY:  HSI ж
  *                                RCC_IT_HSERDY:  HSE ж
  *                                RCC_IT_PLLRDY:  PLL ж
  *                                RCC_IT_CSS:     ʱӰȫϵͳж
  * @  û
  */
void RCC_ClearITPendingBit(uint8_t RCC_IT)
{
  /*  */
  assert_param(IS_RCC_CLEAR_IT(RCC_IT));

  /* ִ RCC_CIR[23:16] λֽڴȡѡжϹλ*/
  *(__IO uint8_t *) CIR_BYTE3_ADDRESS = RCC_IT;
}

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/******************* (C) COPYRIGHT 2009 STMicroelectronics *****END OF FILE****/
